package java.util;

import java.util.function.UnaryOperator;

public interface List<E> extends Collection<E>
{
    public abstract int size();
    
    public abstract boolean isEmpty();
    
    public abstract boolean contains(Object p0);
    
    public abstract Iterator<E> iterator();
    
    public abstract Object[] toArray();
    
    public abstract <T> T[] toArray(T[] p0);
    
    public abstract boolean add(E p0);
    
    public abstract boolean remove(Object p0);
    
    public abstract boolean containsAll(Collection<?> p0);
    
    public abstract boolean addAll(Collection<? extends E> p0);
    
    public abstract boolean addAll(int p0, Collection<? extends E> p1);
    
    public abstract boolean removeAll(Collection<?> p0);
    
    public abstract boolean retainAll(Collection<?> p0);
    
    default public void replaceAll(UnaryOperator<E> unaryOperator) {
        Objects.requireNonNull(unaryOperator);
        ListIterator<Object> listIterator = this.listIterator();
        while (listIterator.hasNext()) {
            listIterator.set(unaryOperator.apply(listIterator.next()));
        }
    }
    
    default public void sort(Comparator<? super E> comparator) {
        Object[] array = this.toArray();
        Arrays.sort(array, comparator);
        ListIterator<Object> listIterator = this.listIterator();
        for (Object o : array) {
            listIterator.next();
            listIterator.set(o);
        }
    }
    
    public abstract void clear();
    
    public abstract boolean equals(Object p0);
    
    public abstract int hashCode();
    
    public abstract E get(int p0);
    
    public abstract E set(int p0, E p1);
    
    public abstract void add(int p0, E p1);
    
    public abstract E remove(int p0);
    
    public abstract int indexOf(Object p0);
    
    public abstract int lastIndexOf(Object p0);
    
    public abstract ListIterator<E> listIterator();
    
    public abstract ListIterator<E> listIterator(int p0);
    
    public abstract List<E> subList(int p0, int p1);
    
    default public Spliterator<E> spliterator() {
        return Spliterators.spliterator(this, 16);
    }
}
